<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.


declare(strict_types=1);

namespace NWCModule\Controller;


use App\itnovum\openITCOCKPIT\Wizard\CreateService;
use App\itnovum\openITCOCKPIT\Wizard\UpdateHost;
use App\Model\Entity\Host;
use App\Model\Table\HostsTable;
use App\Model\Table\HosttemplatesTable;
use App\Model\Table\ServicesTable;
use App\Model\Table\ServicetemplatesTable;
use App\Model\Table\WizardAssignmentsTable;
use Cake\Http\Exception\MethodNotAllowedException;
use Cake\Http\Exception\NotFoundException;
use Cake\ORM\TableRegistry;
use Cake\Utility\Hash;
use itnovum\openITCOCKPIT\Core\AngularJS\Api;
use itnovum\openITCOCKPIT\Core\System\Gearman;
use itnovum\openITCOCKPIT\Core\ValueObjects\User;
use NWCModule\Form\NetworkBasicForm;
use NWCModule\itnovum\openITCOCKPIT\SNMP\SNMPScanNwc;

/**
 * Class WizardsController
 * @package NWCModule\Controller
 */
class WizardsController extends AppController {


    //dependency apt-get install libnet-snmp-perl
    /**
     * @param null $hostId
     */
    public function networkbasic($hostId = null) {
        if (!$this->isApiRequest()) {
            //Only ship HTML template for angular
            return;
        }


        /** @var WizardAssignmentsTable $WizardAssignmentsTable */
        $WizardAssignmentsTable = TableRegistry::getTableLocator()->get('WizardAssignments');
        $wizards = $WizardAssignmentsTable->getAvailableWizards($this->PERMISSIONS);
        /** @var ServicesTable $ServicesTable */
        $ServicesTable = TableRegistry::getTableLocator()->get('Services');
        /** @var HostsTable $HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');

        if ($this->request->is('get') && $this->isAngularJsRequest()) {
            if (!$HostsTable->existsById($hostId)) {
                throw new NotFoundException();
            }
            //Return nwc wizard data
            $servicetemplates = [];
            $wizardAssignments = [];
            $networkWizardData = $wizards['network'] ?? null;
            if ($networkWizardData) {
                if ($WizardAssignmentsTable->existsByUuidAndTypeId($networkWizardData['uuid'], $networkWizardData['type_id'])) {
                    $wizardAssignments = Hash::merge($networkWizardData, $WizardAssignmentsTable->getWizardByUuidForEdit($networkWizardData['uuid']));
                }
            }

            if (!empty($wizardAssignments['servicetemplates']['_ids'])) {
                /** @var ServicetemplatesTable $ServicetemplatesTable */
                $ServicetemplatesTable = TableRegistry::getTableLocator()->get('Servicetemplates');
                $excludedUuids = [
                    'f08d15ab-4de9-4a2d-8d3c-3f5bf13c21e7', // NETWORK_NWC_LIST_INTERFACES - for interfaces scan only
                    'de5e3045-3011-45d8-8ac6-bc5fbb3d396d'  // NETWORK_NWC_INTERFACE_HEALTH - only used by creating of interface services
                ];

                $servicetemplates = $ServicetemplatesTable->getServicetemplatesFoWizardDeploy(
                    $wizardAssignments['servicetemplates']['_ids'],
                    $this->MY_RIGHTS,
                    $excludedUuids
                );
            }

            $interfaceServicetemplate = $ServicetemplatesTable->getServicetemplateFoWizardDeployInterfaces($this->MY_RIGHTS);

            $servicesNamesForExistCheck = $ServicesTable->getServiceNamesByHostIdForWizard($hostId);


            $this->set('servicetemplates', $servicetemplates);
            $this->set('servicesNamesForExistCheck', $servicesNamesForExistCheck);
            $this->set('interfaceServicetemplate', $interfaceServicetemplate);

            $this->viewBuilder()->setOption('serialize', [
                'servicetemplates',
                'interfaceServicetemplate',
                'servicesNamesForExistCheck'
            ]);

            return;
        }
        if ($this->request->is('post') && $this->isAngularJsRequest()) {
            $hostId = $this->request->getData('host_id', 0);
            if (!$HostsTable->existsById($hostId)) {
                throw new NotFoundException();
            }

            $host = $HostsTable->getHostByIdForPermissionCheck($hostId);
            if (!$this->allowedByContainerId($host->getContainerIds())) {
                $this->render403();
                return;
            }


            $NetworkBasicForm = new NetworkBasicForm();
            $data = $this->request->getData(null, []);
            $NetworkBasicForm->execute($data);

            if (!empty($NetworkBasicForm->getErrors())) {
                $this->response = $this->response->withStatus(400);
                $this->set('error', $NetworkBasicForm->getErrors());
                $this->viewBuilder()->setOption('serialize', ['error']);
                return;
            }

            //Wizard validation successfully - update host and create services
            $User = new User($this->getUser());

            /** @var HosttemplatesTable $HosttemplatesTable */
            $HosttemplatesTable = TableRegistry::getTableLocator()->get('Hosttemplates');

            // Get host and host template for merge
            $host = $HostsTable->getHostForEdit($hostId);
            $hosttemplate = $HosttemplatesTable->getHosttemplateForDiff($host['Host']['hosttemplate_id']);

            //Update custom variables
            $SnmpScan = new SNMPScanNwc($data);
            $hostCustomVariablestoCheck = [
                'SNMP_SETTINGS' => 'snmpsettings'
            ];

            //Update host with changelog
            $hostEntity = UpdateHost::save(
                $host,
                $hosttemplate,
                ['snmpsettings' => $SnmpScan->getSnmpString()],
                $hostCustomVariablestoCheck,
                $HostsTable,
                $User
            );

            if ($hostEntity instanceof Host) {
                //Host updated successfully - create services

                /** @var ServicetemplatesTable $ServicetemplatesTable */
                $ServicetemplatesTable = TableRegistry::getTableLocator()->get('Servicetemplates');

                // Save default NWC services
                CreateService::saveMany(
                    $hostEntity,
                    $data['services'],
                    $data,
                    [],
                    $HostsTable,
                    $HosttemplatesTable,
                    $ServicesTable,
                    $ServicetemplatesTable,
                    $User
                );

                // Save NWC interfaces services
                CreateService::saveMany(
                    $hostEntity,
                    $data['interfaces'],
                    $data,
                    [],
                    $HostsTable,
                    $HosttemplatesTable,
                    $ServicesTable,
                    $ServicetemplatesTable,
                    $User
                );
            }
        }
        return;
    }

    /****************************
     *       AJAX METHODS       *
     ****************************/

    /**
     * @param null $hostId
     */
    public function executeSNMPDiscovery($hostId = null) {
        if (!$this->isAngularJsRequest()) {
            throw new MethodNotAllowedException();
        }

        /** @var  HostsTable $HostsTable */
        $HostsTable = TableRegistry::getTableLocator()->get('Hosts');
        if (!$HostsTable->existsById($hostId)) {
            throw new NotFoundException(__('Invalid host'));
        }

        $host = $HostsTable->getHostById($hostId);

        $NetworkBasicForm = new NetworkBasicForm();
        $data = $this->request->getData(null, []);
        $NetworkBasicForm->execute($data);

        if (!empty($NetworkBasicForm->getErrors())) {
            $this->response = $this->response->withStatus(400);
            $this->set('error', $NetworkBasicForm->getErrors());
            $this->viewBuilder()->setOption('serialize', ['error']);
            return;
        }

        /** @var Gearman $GearmanClient */
        $GearmanClient = new Gearman();
        $result = $GearmanClient->send('WizardNwcInterfaceList', [
            'data'         => $data,
            'host_address' => $host->get('address')
        ]);
        if ($result['success'] === true) {
            $interfaces = $result['interfaces'];
            $interfaces = Api::makeItJavaScriptAble($interfaces);
            $this->set('interfaces', $interfaces);
            $this->viewBuilder()->setOption('serialize', ['interfaces']);
        } else {
            $this->response = $this->response->withStatus(400);
            $this->set('error', [
                'snmpcommand' => [
                    __('SNMP discovery is not possible. Please check your SNMP credentials')
                ]
            ]);
            $this->viewBuilder()->setOption('serialize', ['error']);
            return;
        }
    }
}
