<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.


namespace NWCModule\itnovum\openITCOCKPIT\SNMP;


use Cake\Filesystem\File;
use itnovum\openITCOCKPIT\Core\SNMP\SNMPScan;
use Symfony\Component\Filesystem\Exception\FileNotFoundException;


/**
 * Class SNMPScanNwc
 * @package NWCModule\itnovum\openITCOCKPIT\SNMP
 */
class SNMPScanNwc extends SNMPScan {

    /**
     * @param $hostAddress
     * @return array
     */
    public function executeSnmpDiscovery($hostAddress) {
        if ($this->snmpVersion === '3') {
            $credentialArray = $this->generateV3SnmpAsArray();
        } else {
            $credentialArray = $this->generateV1OrV2SnmpAsArray();
        }

        $checkCommandFilePath = '/opt/openitc/nagios/libexec/check_nwc_health';
        $file = new File($checkCommandFilePath);
        if (!$file->exists()) {
            throw new FileNotFoundException('Command for interface discovery is missing');
        }
        $processOptions = [
            'sudo',
            '-u',
            'nagios',
            '-g',
            'www-data',
            $checkCommandFilePath,
            '--hostname',
            $hostAddress
        ];

        foreach ($credentialArray as $credential) {
            $processOptions[] = $credential;
        }

        $processOptions[] = '--mode';
        $processOptions[] = 'list-interfaces';
        $processOptions[] = '--statefilesdir';
        $processOptions[] = '/var/tmp/check_nwc_health';

        exec(implode(' ', $processOptions), $outputAsArray, $rc);

        //$process = new Process($processOptions);
        //$process->run();

        // executes after the command finishes
        if ($rc > 0) {
            throw new \RuntimeException(implode(' ', $outputAsArray));
        }

        $interfaces = [];
        if (in_array('OK - have fun', $outputAsArray)) {
            foreach ($outputAsArray as $key => $interfaceName) {
                if ($interfaceName === 'OK - have fun') {
                    break;
                }
                $interfaceDetails = preg_split('/\s+/', $interfaceName, 2);
                if (!empty($interfaceDetails[0]) && !empty($interfaceDetails[1])) {
                    $interfaces[] = [
                        'number' => $interfaceDetails[0],
                        'name'   => $interfaceDetails[1]
                    ];
                }
            }
        }
        return $interfaces;
    }
}
