angular.module('openITCOCKPIT')
    .controller('WizardsNetworkBasicController', function($scope, $http, $stateParams, QueryStringService, NotyService, RedirectService){
        $scope.hostId = QueryStringService.getStateValue($stateParams, 'hostId', false);
        /** public vars **/
        $scope.init = true;
        $scope.disableSubmit = false;

        $scope.showPlainPassword = false;
        $scope.showPlainEncryptionPassword = false;
        $scope.executingDiscovery = false;

        $scope.filteredItems = [];
        $scope.filteredInterfaces = [];

        $scope.post = {
            snmpVersion: '2',
            snmpCommunity: '',
            securityLevel: '1',
            authProtocol: 'md5',
            securityName: '',
            authPassword: '',
            privacyProtocol: 'des',
            privacyPassword: '',
            services: [],
            interfaces: []
        };

        $scope.load = function(){
            $http.get("/nwc_module/wizards/networkbasic/" + $scope.hostId + ".json", {
                params: {
                    'angular': true
                }
            }).then(function(result){
                $scope.servicetemplates = result.data.servicetemplates;
                $scope.servicesNamesForExistCheck = result.data.servicesNamesForExistCheck;

                for(var key in $scope.servicetemplates){
                    $scope.post.services.push(
                        {
                            'host_id': $scope.hostId,
                            'servicetemplate_id': $scope.servicetemplates[key].id,
                            'name': $scope.servicetemplates[key].name,
                            'description': $scope.servicetemplates[key].description,
                            'servicecommandargumentvalues': $scope.servicetemplates[key].servicetemplatecommandargumentvalues,
                            'createService': !(in_array($scope.servicetemplates[key].name, $scope.servicesNamesForExistCheck))
                        });
                }
                $scope.interfaceServicetemplate = result.data.interfaceServicetemplate;
                $scope.init = false;
            });
        };

        $scope.submit = function(){
            $scope.disableSubmit = true;

            var services = [];
            for(var index in $scope.post.services){
                if($scope.post.services[index].createService === true && $scope.filteredItems.includes($scope.post.services[index])){
                    services.push($scope.post.services[index]);
                }
            }
            var interfaces = [];
            for(var index in $scope.post.interfaces){
                if($scope.post.interfaces[index].createService === true && $scope.filteredInterfaces.includes($scope.post.interfaces[index])){
                    interfaces.push($scope.post.interfaces[index]);
                }
            }
            var post = JSON.parse(JSON.stringify($scope.post)); // Remove JS binding
            post.host_id = $scope.hostId;
            post.services = services;
            post.interfaces = interfaces;

            $http.post("/nwc_module/wizards/networkbasic.json?angular=true",
                post
            ).then(function(result){
                $scope.disableSubmit = false;
                NotyService.genericSuccess();
                RedirectService.redirectWithFallback('ServicesNotMonitored');
                $scope.errors = {};
                NotyService.scrollTop();
                console.log('Data saved successfully');
            }, function errorCallback(result){
                $scope.disableSubmit = false;

                NotyService.genericError();
                NotyService.scrollTop();
                if(result.data.hasOwnProperty('error')){
                    $scope.errors = result.data.error;
                }
            });

        };

        $scope.detectColor = function(label){
            if(label.match(/warning/gi)){
                return 'warning';
            }

            if(label.match(/critical/gi)){
                return 'critical';
            }

            return '';
        };

        $scope.startInterfaceDiscovery = function(){
            $scope.post.interfaces = [];

            if($scope.post.snmpVersion > 0){
                $scope.executingDiscovery = true;

                var params = {
                    host_id: $scope.hostId,
                    snmpVersion: $scope.post.snmpVersion,
                    snmpCommunity: $scope.post.snmpCommunity,
                    securityLevel: $scope.post.securityLevel,
                    authProtocol: $scope.post.authProtocol,
                    securityName: $scope.post.securityName,
                    authPassword: $scope.post.authPassword,
                    privacyProtocol: $scope.post.privacyProtocol,
                    privacyPassword: $scope.post.privacyPassword

                };

                $http.post("/nwc_module/wizards/executeSNMPDiscovery/" + $scope.hostId + ".json?angular=true", params)
                    .then(function(result){
                        $scope.executingDiscovery = false;
                        $scope.post.interfaces = [];
                        $scope.errors = {};
                        $scope.interfaces = result.data.interfaces;

                        for(var key in $scope.interfaces){
                            var servicetemplatecommandargumentvalues = JSON.parse(angular.toJson(
                                    $scope.interfaceServicetemplate.servicetemplatecommandargumentvalues
                                )
                            );
                            servicetemplatecommandargumentvalues[0].value = $scope.interfaces[key].value.name;
                            $scope.post.interfaces.push(
                                {
                                    'host_id': $scope.hostId,
                                    'servicetemplate_id': $scope.interfaceServicetemplate.id,
                                    'name': $scope.interfaces[key].value.name,
                                    'description': $scope.interfaces[key].value.number,
                                    'servicecommandargumentvalues': servicetemplatecommandargumentvalues,
                                    'createService': !(in_array($scope.interfaces[key].value.name, $scope.servicesNamesForExistCheck))
                                });
                        }


                    }, function errorCallback(result){
                        $scope.executingDiscovery = false;
                        NotyService.genericError();

                        if(result.data.hasOwnProperty('error')){
                            $scope.errors = result.data.error;
                            if($scope.errors.hasOwnProperty('snmpcommand') === false){
                                NotyService.scrollTop();
                            }
                        }
                    });
            }
        };

        //Fire on page load
        $scope.load();
    });
